@@ -6,7 +6,7 @@ from sklearn.base import BaseEstimator, ClassifierMixin
 from sklearn.utils.validation import check_is_fitted
 from sklearn.pipeline import make_pipeline
 from sklearn.linear_model import RidgeClassifierCV
-from sklearn.preprocessing import StandardScaler
+from convst.transformers._input_transformers import c_StandardScaler
 from convst.transformers import R_DST
 
 from sklearn.metrics import accuracy_score
@@ -27,14 +27,14 @@ class R_DST_Ridge(BaseEstimator, ClassifierMixin):
     ----------
     n_shapelets : int, optional
         Number of shapelets to generate. The default is 10000.
-    shapelet_sizes : array
+    shapelet_sizes : array, optional
         An array of int which indicate the possible absolute shapelet sizes.
         Or an array of float which will give shapelet sizes relative to input length.
         The default is [11]
-    p_norm : float
+    p_norm : float, optional
         A float between 0 and 1 indicating the proportion of shapelets that
         will use a z-normalized distance. The default is 0.8.
-    percentiles : array, shape=(2)
+    percentiles : array, shape=(2), optional
         The two percentiles (between 0 and 100) between which the value of the
         threshold will be sampled during shapelet generation. 
         The default is [5,10].
@@ -43,7 +43,8 @@ class R_DST_Ridge(BaseEstimator, ClassifierMixin):
         The default is None.
     n_jobs : int, optional
         Number of thread used by numba for the computational heavy part
-        of the algortihm. The default is -1 (i.e all available cores).
+        of the algortihm. The default is 1. Change to -1 to use all
+        available cores.
     class_weight : object, optional
         Class weight option of Ridge Classifier, either None, "balanced" or a
         custom dictionnary of weight for each class. The default is None.
@@ -56,30 +57,74 @@ class R_DST_Ridge(BaseEstimator, ClassifierMixin):
         The default is np.logspace(-4,4,10).
     """
     
-    def __init__(self, n_shapelets=10000, shapelet_sizes=[11], p_norm=0.8,
-                 percentiles=[5, 10], n_jobs=-1, random_state=None,
-                 class_weight=None, fit_intercept=True,
-                 alphas=np.logspace(-4,4,10)):
+    def __init__(
+        self, 
+        transform_type='auto',
+        phase_invariance=False,
+        distance='manhattan',
+        alpha=0.5,
+        normalize_output=False,
+        n_samples=1.0,
+        n_shapelets=10_000,
+        shapelet_lengths=[11],
+        proba_norm=0.8,
+        percentiles=[5,10],
+        n_jobs=1,
+        random_state=None,
+        min_len=None,
+        class_weight=None, 
+        fit_intercept=True,
+        alphas_ridge=list(np.logspace(-6,6,20))
+    ):
+        self.alphas_ridge=alphas_ridge
+        self.class_weight=class_weight
+        self.fit_intercept=fit_intercept
+        self.transform_type=transform_type
+        self.phase_invariance=phase_invariance
+        self.distance=distance
+        self.alpha=alpha
+        self.normalize_output=normalize_output
+        self.n_samples=n_samples
+        self.n_shapelets=n_shapelets
+        self.shapelet_lengths=shapelet_lengths
+        self.proba_norm=proba_norm
+        self.percentiles=percentiles
+        self.n_jobs=n_jobs
+        self.random_state=random_state
+        self.min_len=min_len
 
+    def _more_tags(self):
+        return {
+            "capability:variable_length": True,
+            "capability:univariate": True,
+            "capability:multivariate": True,
+            "capability:multithreading": True,
+        }
+
+    def _init_components(self):
         self.classifier = make_pipeline(
-            StandardScaler(with_mean=False),
+            c_StandardScaler(with_mean=True),
             RidgeClassifierCV(
-                alphas=alphas, class_weight=class_weight, 
-                fit_intercept=fit_intercept
+                alphas=self.alphas_ridge,
+                class_weight=self.class_weight, 
+                fit_intercept=self.fit_intercept
             )
         )
         self.transformer = R_DST(
-            n_shapelets=n_shapelets, shapelet_sizes=shapelet_sizes, 
-            p_norm=p_norm, percentiles=percentiles,
-            n_jobs=n_jobs, random_state=random_state
+            transform_type=self.transform_type,
+            phase_invariance=self.phase_invariance,
+            distance=self.distance,
+            alpha=self.alpha,
+            normalize_output=self.normalize_output,
+            n_samples=self.n_samples,
+            n_shapelets=self.n_shapelets,
+            shapelet_lengths=self.shapelet_lengths,
+            proba_norm=self.proba_norm,
+            percentiles=self.percentiles,
+            n_jobs=self.n_jobs,
+            random_state=self.random_state,
+            min_len=self.min_len 
         )
-        
-        
-    def __repr__(self):
-        return self.transformer.__repr__() + " " + self.classifier.__repr__()
-    
-    def _more_tags(self):
-        return ["R_DST"]
     
     def fit(self, X, y):
         """
@@ -95,6 +140,7 @@ class R_DST_Ridge(BaseEstimator, ClassifierMixin):
         y : array, shape=(n_samples)
             Class of the input time series.
         """
+        self._init_components()
         self.transformer = self.transformer.fit(X, y)
         self.classifier = self.classifier.fit(self.transformer.transform(X), y)
         return self
@@ -116,8 +162,7 @@ class R_DST_Ridge(BaseEstimator, ClassifierMixin):
 
         """
         check_is_fitted(self, ['classifier'])
-        X = self.transformer.transform(X)
-        return self.classifier.predict(X)
+        return self.classifier.predict(self.transformer.transform(X))
     
     def score(self, X, y):
         """
@@ -139,4 +184,4 @@ class R_DST_Ridge(BaseEstimator, ClassifierMixin):
 
         """
         preds = self.predict(X)
-        return accuracy_score(y, preds)
\ No newline at end of file
+        return accuracy_score(y, preds)
