@@ -2,11 +2,69 @@
 
 import numpy as np
 from sktime.datasets import (load_UCR_UEA_dataset, 
-    load_from_tsfile_to_dataframe, load_from_arff_to_dataframe
+    load_from_tsfile_to_dataframe, load_from_arff_to_dataframe,
+    
+)
+from sktime.datatypes._panel._convert import (from_multiindex_to_dflist,
+    from_nested_to_multi_index
 )
-from sktime.datatypes._panel._convert import from_nested_to_3d_numpy
-
 from sklearn.preprocessing import LabelEncoder
+from numba import njit, prange
+
+
+def _custom_from_nested_to_3d_numpy(X):
+    X = from_multiindex_to_dflist(from_nested_to_multi_index(X))
+    if all([X[i].shape[0] == X[0].shape[0] for i in range(len(X))]):
+        return np.array([X[i].values.T for i in range(len(X))])
+    else:
+        return [X[i].values.T for i in range(len(X))]
+    
+
+@njit(cache=True)
+def z_norm_3D(X):
+    """
+    Z normalise a time series dataset assumed to be of even length. A small value
+    is added to the standard deviation for all samples and features to avoid
+    0 division.
+
+    Parameters
+    ----------
+    X : array, shape=(n_samples, n_features, n_timestamps)
+        Input numerical array to z-normalise
+
+    Returns
+    -------
+    X : array, shape=(n_samples, n_features, n_timestamps)
+        Z-normalised array
+
+    """
+    for i_x in prange(X.shape[0]):
+        for i_ft in prange(X.shape[1]):
+            X[i_x, i_ft] = (X[i_x, i_ft] - X[i_x, i_ft].mean())/(X[i_x, i_ft].std() + 1e-8)
+    return X
+    
+def z_norm_3D_list(X):
+    """
+    Z normalise a time series dataset assumed to be of even length. A small value
+    is added to the standard deviation for all samples and features to avoid
+    0 division.
+
+    Parameters
+    ----------
+    X : array, shape=(n_samples, n_features, n_timestamps)
+        Input numerical array to z-normalise
+
+    Returns
+    -------
+    X : array, shape=(n_samples, n_features, n_timestamps)
+        Z-normalised array
+
+    """
+    for i_x in range(len(X)):
+        for i_ft in range(len(X[i_x])):
+            X[i_x][i_ft] = (X[i_x][i_ft] - X[i_x][i_ft].mean())/(X[i_x][i_ft].std() + 1e-8)
+    return X
+    
 
 def load_sktime_dataset_split(name, normalize=True):
     """
@@ -41,8 +99,8 @@ def load_sktime_dataset_split(name, normalize=True):
     X_test, y_test = load_UCR_UEA_dataset(name, return_X_y=True, split='test')
 
     #Convert pandas DataFrames to numpy arrays
-    X_train = from_nested_to_3d_numpy(X_train)
-    X_test = from_nested_to_3d_numpy(X_test)
+    X_train = _custom_from_nested_to_3d_numpy(X_train)
+    X_test = _custom_from_nested_to_3d_numpy(X_test)
 
     #Convert class labels to make sure they are between 0,n_classes
     le = LabelEncoder().fit(y_train)
@@ -50,12 +108,12 @@ def load_sktime_dataset_split(name, normalize=True):
     y_test = le.transform(y_test)
 
     #Z-Normalize the data
-    if normalize:
-        X_train = (X_train - X_train.mean(axis=-1, keepdims=True)) / (
-            X_train.std(axis=-1, keepdims=True) + 1e-8)
-        X_test = (X_test - X_test.mean(axis=-1, keepdims=True)) / (
-            X_test.std(axis=-1, keepdims=True) + 1e-8)
-
+    if normalize and not isinstance(X_train, list):
+        X_train = z_norm_3D(X_train)
+        X_test = z_norm_3D(X_test)
+    if normalize and isinstance(X_train, list):
+        X_train = z_norm_3D_list(X_train)
+        X_test = z_norm_3D_list(X_test)
     return X_train, X_test, y_train, y_test, le
 
 
@@ -92,8 +150,8 @@ def load_sktime_arff_file(path, normalize=True):
     X_test, y_test = load_from_arff_to_dataframe(path+'_TEST.arff')
 
     #Convert pandas DataFrames to numpy arrays
-    X_train = from_nested_to_3d_numpy(X_train)
-    X_test = from_nested_to_3d_numpy(X_test)
+    X_train = _custom_from_nested_to_3d_numpy(X_train)
+    X_test = _custom_from_nested_to_3d_numpy(X_test)
 
     #Convert class labels to make sure they are between 0,n_classes
     le = LabelEncoder().fit(y_train)
@@ -146,8 +204,8 @@ def load_sktime_arff_file_resample_id(path, rs_id, normalize=True):
     X_test, y_test = load_from_arff_to_dataframe(path+'_{}_TEST.arff'.format(rs_id))
 
     #Convert pandas DataFrames to numpy arrays
-    X_train = from_nested_to_3d_numpy(X_train)
-    X_test = from_nested_to_3d_numpy(X_test)
+    X_train = _custom_from_nested_to_3d_numpy(X_train)
+    X_test = _custom_from_nested_to_3d_numpy(X_test)
 
     #Convert class labels to make sure they are between 0,n_classes
     le = LabelEncoder().fit(y_train)
@@ -196,8 +254,8 @@ def load_sktime_ts_file(path, normalize=True):
     X_test, y_test = load_from_tsfile_to_dataframe(path+'_TEST.ts')
 
     #Convert pandas DataFrames to numpy arrays
-    X_train = from_nested_to_3d_numpy(X_train)
-    X_test = from_nested_to_3d_numpy(X_test)
+    X_train = _custom_from_nested_to_3d_numpy(X_train)
+    X_test = _custom_from_nested_to_3d_numpy(X_test)
 
     #Convert class labels to make sure they are between 0,n_classes
     le = LabelEncoder().fit(y_train)
@@ -239,7 +297,7 @@ def load_sktime_dataset(name, normalize=True):
     X, y = load_UCR_UEA_dataset(name, return_X_y=True)
 
     #Convert pandas DataFrames to numpy arrays
-    X = from_nested_to_3d_numpy(X)
+    X = _custom_from_nested_to_3d_numpy(X)
 
     #Convert class labels to make sure they are between 0,n_classes
     le = LabelEncoder().fit(y)
@@ -252,8 +310,74 @@ def load_sktime_dataset(name, normalize=True):
 
     return X, y, le
 
-
 def return_all_dataset_names():
+    return np.concatenate((
+        return_all_univariate_dataset_names(),
+        return_all_multivariate_dataset_names(),
+        return_all_variable_univariate_dataset_names(),
+        return_all_variable_multivariate_dataset_names()
+    ))
+
+def return_all_multivariate_dataset_names():
+    
+    return np.asarray([
+        "ArticularyWordRecognition",
+        "AtrialFibrillation",
+        "BasicMotions",
+        "Cricket",
+        "DuckDuckGeese",
+        "EigenWorms",
+        "Epilepsy",
+        "EthanolConcentration",
+        "ERing",
+        "FaceDetection",
+        "FingerMovements",
+        "HandMovementDirection",
+        "Handwriting",
+        "Heartbeat",
+        "Libras",
+        "LSST",
+        "MotorImagery",
+        "NATOPS",
+        "PenDigits",
+        "PEMS-SF",
+        "PhonemeSpectra",
+        "RacketSports",
+        "SelfRegulationSCP1",
+        "SelfRegulationSCP2",
+        "StandWalkJump",
+        "UWaveGestureLibrary"
+    ])
+
+def return_all_variable_multivariate_dataset_names():
+    return np.asarray([
+        "AsphaltObstaclesCoordinates",
+        "AsphaltPavementTypeCoordinates",
+        "AsphaltRegularityCoordinates",
+        "CharacterTrajectories",
+        "InsectWingbeat",
+        "JapaneseVowels",
+        "SpokenArabicDigits"
+    ])
+
+
+def return_all_variable_univariate_dataset_names():
+    return np.asarray([
+        "AllGestureWiimoteX",
+        "AllGestureWiimoteY",
+        "AllGestureWiimoteZ",
+        "GestureMidAirD1",
+        "GestureMidAirD2",
+        "GestureMidAirD3",
+        "GesturePebbleZ1",
+        "GesturePebbleZ2",
+        "PickupGestureWiimoteZ",
+        "PLAID",
+        "ShakeGestureWiimoteZ"
+    ])
+
+
+def return_all_univariate_dataset_names():
     """
     Return the names of the 112 univariate datasets of the UCR archive.
 
@@ -275,13 +399,11 @@ def return_all_dataset_names():
             "CBF",
             "Chinatown",
             "ChlorineConcentration",
-            "CinCECGTorso",
             "Coffee",
             "Computers",
             "CricketX",
             "CricketY",
             "CricketZ",
-            "Crop",
             "DiatomSizeReduction",
             "DistalPhalanxOutlineAgeGroup",
             "DistalPhalanxOutlineCorrect",
@@ -293,7 +415,6 @@ def return_all_dataset_names():
             "ElectricDevices",
             "EOGHorizontalSignal",
             "EOGVerticalSignal",
-            "EthanolLevel",
             "FaceAll",
             "FaceFour",
             "FacesUCR",
@@ -308,7 +429,6 @@ def return_all_dataset_names():
             "GunPointMaleVersusFemale",
             "GunPointOldVersusYoung",
             "Ham",
-            "HandOutlines",
             "Haptics",
             "Herring",
             "HouseTwenty",
@@ -334,17 +454,12 @@ def return_all_dataset_names():
             "OliveOil",
             "OSULeaf",
             "PhalangesOutlinesCorrect",
-            "Phoneme",
-            "PigAirwayPressure",
-            "PigArtPressure",
-            "PigCVP",
             "Plane",
             "PowerCons",
             "ProximalPhalanxOutlineAgeGroup",
             "ProximalPhalanxOutlineCorrect",
             "ProximalPhalanxTW",
             "RefrigerationDevices",
-            "Rock",
             "ScreenType",
             "SemgHandGenderCh2",
             "SemgHandMovementCh2",
@@ -355,7 +470,6 @@ def return_all_dataset_names():
             "SmoothSubspace",
             "SonyAIBORobotSurface1",
             "SonyAIBORobotSurface2",
-            "StarLightCurves",
             "Strawberry",
             "SwedishLeaf",
             "Symbols",
@@ -375,4 +489,15 @@ def return_all_dataset_names():
             "WordSynonyms",
             "Worms",
             "WormsTwoClass",
-            "Yoga"])
+            "Yoga",
+            "Phoneme",
+            "PigAirwayPressure",
+            "PigArtPressure",
+            "PigCVP",
+            "Crop",
+            "StarLightCurves",
+            "Rock",
+            "HandOutlines",
+            "CinCECGTorso",
+            "EthanolLevel"
+            ])
